/*
 * MikroSDK - MikroE Software Development Kit
 * Copyright (c) 2019, MikroElektronika - www.mikroe.com
 * All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/*!
 * \file
 *
 */

#include "excelonlp.h"

// ------------------------------------------------------------- PRIVATE MACROS 

#define EXCELONLP_DUMMY 0

// ------------------------------------------------ PUBLIC FUNCTION DEFINITIONS

void excelonlp_cfg_setup ( excelonlp_cfg_t *cfg )
{
    // Communication gpio pins 

    cfg->sck = HAL_PIN_NC;
    cfg->miso = HAL_PIN_NC;
    cfg->mosi = HAL_PIN_NC;
    cfg->cs = HAL_PIN_NC;

    // Additional gpio pins

    cfg->wp = HAL_PIN_NC;

    cfg->spi_speed = SPI_MEDIUM; 
    cfg->spi_mode = SPI_MODE_0;
}

EXCELONLP_RETVAL excelonlp_init ( excelonlp_t *ctx, excelonlp_cfg_t *cfg )
{
    spi_master_config_t spi_cfg;

    spi_master_configure_default( &spi_cfg );
    spi_cfg.mode        = cfg->spi_mode;
    spi_cfg.speed       = cfg->spi_speed;
    spi_cfg.pins.sck    = cfg->sck;
    spi_cfg.pins.miso   = cfg->miso;
    spi_cfg.pins.mosi   = cfg->mosi;
    spi_cfg.chip_select = cfg->cs;

    if (  spi_master_open( &ctx->spi, &spi_cfg ) != SPI_SUCCESS )
    {
        return EXCELONLP_INIT_ERROR;
    }

    spi_master_set_dummy_data( &ctx->spi, EXCELONLP_DUMMY );

    // Output pins 

    digital_out_init( &ctx->wp, cfg->wp );

    return EXCELONLP_OK;
}

void excelonlp_generic_transfer ( excelonlp_t *ctx, spi_master_transfer_data_t *block )
{
    spi_master_start( &ctx->spi ) ;
    spi_master_transfer( &ctx->spi, block );
    spi_master_stop( &ctx->spi );    
}

void excelonlp_send_command ( excelonlp_t *ctx, uint8_t opcode )
{
    spi_master_start( &ctx->spi );
    spi_master_write( &ctx->spi, opcode );
    spi_master_stop( &ctx->spi );    
}

void excelonlp_read_data ( excelonlp_t *ctx, uint8_t opcode, uint8_t *out_buf, uint8_t n_data)
{
    spi_master_transfer_data_t block;
    
    block.tx_buffer = &opcode;
    block.rx_buffer = out_buf;
    block.tx_length = 1;
    block.rx_length = n_data;

    excelonlp_generic_transfer( ctx, &block );
}

void excelonlp_write_memory_data ( excelonlp_t *ctx, uint8_t opcode, uint32_t addr, uint8_t c_data)
{
    spi_master_start( &ctx->spi );
    spi_master_write( &ctx->spi, opcode );
    spi_master_write( &ctx->spi, ( uint8_t ) ( ( addr >> 16 ) & 0x000000FF ) );
    spi_master_write( &ctx->spi, ( uint8_t ) ( ( addr >> 8 ) & 0x000000FF ) );
    spi_master_write( &ctx->spi, ( uint8_t ) ( addr & 0x000000FF ) );
    spi_master_write( &ctx->spi, c_data );
    spi_master_stop( &ctx->spi );
}

uint8_t excelonlp_read_memory_data ( excelonlp_t *ctx, uint8_t opcode, uint32_t addr )
{
    uint8_t write_reg[ 5 ];
    uint8_t read_reg[ 5 ];
    spi_master_transfer_data_t block;
    
    block.tx_buffer = write_reg;
    block.rx_buffer = read_reg;
    block.tx_length = 4;
    block.rx_length = 5;

    write_reg[ 0 ] = ( &ctx->spi, opcode );
    write_reg[ 1 ] = ( &ctx->spi, ( uint8_t ) ( ( addr >> 16 ) & 0x000000FF ) );
    write_reg[ 2 ] = ( &ctx->spi, ( uint8_t ) ( ( addr >> 8 ) & 0x000000FF ) );
    write_reg[ 3 ] = ( &ctx->spi, ( uint8_t ) ( addr & 0x000000FF ) );
    
    excelonlp_generic_transfer( ctx, &block );
     
    return read_reg[ 4 ];
}

// ------------------------------------------------------------------------- END

